@@ -1,6 +1,6 @@
 #===============================================================================
 #
-# $Id: AuthCookieDBI.pm,v 1.44 2008/11/28 23:44:31 matisse Exp $
+# $Id: AuthCookieDBI.pm,v 1.48 2009/04/26 17:33:26 matisse Exp $
 #
 # Apache2::AuthCookieDBI
 #
@@ -9,7 +9,7 @@
 # See end of this file for Copyright notices.
 #
 # Author:  Jacob Davies <jacob@well.com>
-# Maintainer: Matisse Enzer <matisse@matisse.net> (as of version 2.0)
+# Maintainer: Matisse Enzer <matisse@cpan.org> (as of version 2.0)
 #
 #
 # This library is free software; you can redistribute it and/or
@@ -33,8 +33,7 @@ package Apache2::AuthCookieDBI;
 use strict;
 use warnings;
 use 5.004;
-use vars qw( $VERSION );
-$VERSION = '2.11';
+our $VERSION = 2.12;
 
 use Apache2::AuthCookie;
 use base qw( Apache2::AuthCookie );
@@ -190,28 +189,34 @@ and the login form is shown again.
 # cached cipher hasn't been created, & decrypt the session key.
 sub _get_cipher_type {
     my ( $dbi_encryption_type, $auth_name, $secret_key ) = @_;
-    $dbi_encryption_type = lc $dbi_encryption_type;
+    my $lc_encryption_type = lc $dbi_encryption_type;
 
     my %cipher_type = (
         des => sub {
             return $CIPHERS{"des:$auth_name"}
-              || Crypt::CBC->new( $secret_key, 'DES' );
+                || Crypt::CBC->new( $secret_key, 'DES' );
         },
         idea => sub {
             return $CIPHERS{"idea:$auth_name"}
-              || Crypt::CBC->new( $secret_key, 'IDEA' );
+                || Crypt::CBC->new( $secret_key, 'IDEA' );
         },
         blowfish => sub {
             return $CIPHERS{"blowfish:$auth_name"}
-              || Crypt::CBC->new( $secret_key, 'Blowfish' );
+                || Crypt::CBC->new( $secret_key, 'Blowfish' );
         },
         blowfish_pp => sub {
             return $CIPHERS{"blowfish_pp:$auth_name"}
-              || Crypt::CBC->new( $secret_key, 'Blowfish_PP' );
+                || Crypt::CBC->new( $secret_key, 'Blowfish_PP' );
         },
     );
-    my $type = $cipher_type{$dbi_encryption_type}->();
-    return $type;
+    my $code_ref = $cipher_type{$lc_encryption_type}
+        || Carp::confess("Unsupported encryption type: '$dbi_encryption_type'");
+    my $cbc_object = $code_ref->();
+
+    # Cache the object. Caught bug where we were not, thanks to unit tests.
+    $CIPHERS{"$lc_encryption_type:$auth_name"} = $cbc_object;
+
+    return $cbc_object;
 }
 
 sub _encrypt_session_key {
@@ -223,25 +228,25 @@ sub _encrypt_session_key {
     my %encryption_handlers = (
         none => sub { return $session_key },
         des  => sub {
-            $CIPHERS{"des:$auth_name"} ||=
-              Crypt::CBC->new( $secret_key, 'DES' );
+            $CIPHERS{"des:$auth_name"}
+                ||= Crypt::CBC->new( $secret_key, 'DES' );
             return $CIPHERS{"des:$auth_name"}->encrypt_hex($session_key);
         },
         idea => sub {
-            $CIPHERS{"idea:$auth_name"} ||=
-              Crypt::CBC->new( $secret_key, 'IDEA' );
+            $CIPHERS{"idea:$auth_name"}
+                ||= Crypt::CBC->new( $secret_key, 'IDEA' );
             return $CIPHERS{"idea:$auth_name"}->encrypt_hex($session_key);
         },
         blowfish => sub {
-            $CIPHERS{"blowfish:$auth_name"} ||=
-              Crypt::CBC->new( $secret_key, 'Blowfish' );
+            $CIPHERS{"blowfish:$auth_name"}
+                ||= Crypt::CBC->new( $secret_key, 'Blowfish' );
             return $CIPHERS{"blowfish:$auth_name"}->encrypt_hex($session_key);
         },
         blowfish_pp => sub {
-            $CIPHERS{"blowfish_pp:$auth_name"} ||=
-              Crypt::CBC->new( $secret_key, 'Blowfish_PP' );
+            $CIPHERS{"blowfish_pp:$auth_name"}
+                ||= Crypt::CBC->new( $secret_key, 'Blowfish_PP' );
             return $CIPHERS{"blowfish_pp:$auth_name"}
-              ->encrypt_hex($session_key);
+                ->encrypt_hex($session_key);
         },
     );
     my $encrypted_key = $encryption_handlers{$dbi_encryption_type}->();
@@ -255,9 +260,8 @@ sub _log_not_set {
     my ( $r, $variable ) = @_;
     my $auth_name = $r->auth_name;
     return $r->log_error(
-        "Apache2::AuthCookieDBI: $variable not set for auth realm
-$auth_name", $r->uri
-    );
+        "Apache2::AuthCookieDBI: $variable not set for auth realm $auth_name",
+        $r->uri );
 }
 
 #-------------------------------------------------------------------------------
@@ -282,13 +286,13 @@ my %CONFIG_DEFAULT = (
     DBI_Password        => undef,
     DBI_UsersTable      => 'users',
     DBI_UserField       => 'user',
-    DBI_passwordfield   => 'password',
-    DBI_crypttype       => 'none',
-    DBI_groupstable     => 'groups',
-    DBI_groupfield      => 'grp',
-    DBI_groupuserfield  => 'user',
-    DBI_encryptiontype  => 'none',
-    DBI_sessionlifetime => '00-24-00-00',
+    DBI_PasswordField   => 'password',
+    DBI_CryptType       => 'none',
+    DBI_GroupsTable     => 'groups',
+    DBI_GroupField      => 'grp',
+    DBI_GroupUserField  => 'user',
+    DBI_EncryptionType  => 'none',
+    DBI_SessionLifetime => '00-24-00-00',
     DBI_sessionmodule   => 'none',
 );
 
@@ -298,17 +302,17 @@ sub _dbi_config_vars {
     my %c;    # config variables hash
     foreach my $variable ( keys %CONFIG_DEFAULT ) {
         my $value_from_config = _dir_config_var( $r, $variable );
-        $c{$variable} =
-          defined $value_from_config
-          ? $value_from_config
-          : $CONFIG_DEFAULT{$variable};
+        $c{$variable}
+            = defined $value_from_config
+            ? $value_from_config
+            : $CONFIG_DEFAULT{$variable};
         if ( !defined $c{$variable} ) {
             _log_not_set( $r, $variable );
         }
     }
 
     # If we used encryption we need to pull in Crypt::CBC.
-    if ( $c{DBI_encryptiontype} ne 'none' ) {
+    if ( $c{'DBI_EncryptionType'} ne 'none' ) {
         require Crypt::CBC;
     }
 
@@ -500,15 +504,15 @@ sub _dbi_connect {
     my %c = _dbi_config_vars $r;
 
     # get the crypted password from the users database for this user.
-    my $dbh =
-      DBI->connect_cached( $c{DBI_DSN}, $c{DBI_user}, $c{DBI_password} );
+    my $dbh = DBI->connect_cached( $c{'DBI_DSN'}, $c{'DBI_User'},
+        $c{'DBI_Password'} );
     if ( defined $dbh ) {
         return $dbh;
     }
     else {
         my $auth_name = $r->auth_name;
         $r->log_error(
-"Apache2::AuthCookieDBI: couldn't connect to $c{ DBI_DSN } for auth realm $auth_name",
+            "Apache2::AuthCookieDBI: couldn't connect to $c{'DBI_DSN'} for auth realm $auth_name",
             $r->uri
         );
         my ( $pkg, $file, $line, $sub ) = caller(1);
@@ -527,9 +531,9 @@ sub _get_crypted_password {
     my $dbh = _dbi_connect($r) || return;
 
     my $sth = $dbh->prepare_cached( <<"EOS" );
-SELECT $c->{ DBI_passwordfield }
-FROM $c->{ DBI_userstable }
-WHERE $c->{ DBI_userfield } = ?
+SELECT $c->{ DBI_PasswordField }
+FROM $c->{ DBI_UsersTable }
+WHERE $c->{ DBI_UserField } = ?
 EOS
     $sth->execute($user);
     my ($crypted_password) = $sth->fetchrow_array;
@@ -539,7 +543,7 @@ EOS
     else {
         my $auth_name = $r->auth_name;
         $r->log_error(
-"Apache2::AuthCookieDBI: couldn't select password from $c->{ DBI_DSN }, $c->{ DBI_userstable }, $c->{ DBI_userfield } for user $user for auth realm $auth_name",
+            "Apache2::AuthCookieDBI: couldn't select password from $c->{ DBI_DSN }, $c->{ DBI_UsersTable }, $c->{ DBI_UserField } for user $user for auth realm $auth_name",
             $r->uri
         );
         return;
@@ -556,10 +560,10 @@ sub _get_new_session {
     my $dbh = _dbi_connect($r);
     my %session;
     tie %session, $session_module, undef,
-      +{
+        +{
         Handle     => $dbh,
         LockHandle => $dbh,
-      };
+        };
 
     $session{user}       = $user;
     $session{extra_data} = $extra_data;
@@ -607,7 +611,7 @@ The default implementation returns an empty string.
 =cut
 
 sub extra_session_info {
-    my ( $self, $r, @credentials ) = @_;
+    my ( $class, $r, @credentials ) = @_;
 
     return $EMPTY_STRING;
 }
@@ -618,14 +622,14 @@ sub extra_session_info {
 # If there is a problem, return a bogus session key.
 
 sub authen_cred {
-    my ( $self, $r,        @credentials ) = @_;
-    my ( $user, $password, @extra_data )  = @credentials;
+    my ( $class, $r,        @credentials ) = @_;
+    my ( $user,  $password, @extra_data )  = @credentials;
     my $auth_name = $r->auth_name;
     ( $user, $password ) = _defined_or_empty( $user, $password );
 
     if ( !length $user ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: no username supplied for auth realm $auth_name",
+            "Apache2::AuthCookieDBI: no username supplied for auth realm $auth_name",
             $r->uri
         );
         return;
@@ -633,7 +637,7 @@ sub authen_cred {
 
     if ( !length $password ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: no password supplied for auth realm $auth_name",
+            "Apache2::AuthCookieDBI: no password supplied for auth realm $auth_name",
             $r->uri
         );
         return;
@@ -646,17 +650,17 @@ sub authen_cred {
     my $crypted_password = _get_crypted_password( $r, $user, \%c );
 
     # now return unless the passwords match.
-    my $crypt_type = lc $c{DBI_crypttype};
+    my $crypt_type = lc $c{'DBI_CryptType'};
     if ( !_check_password( $password, $crypted_password, $crypt_type ) ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: $crypt_type passwords didn't match for user $user for auth realm $auth_name",
+            "Apache2::AuthCookieDBI: $crypt_type passwords didn't match for user $user for auth realm $auth_name",
             $r->uri
         );
         return;
     }
 
     # Create the expire time for the ticket.
-    my $expire_time = _get_expire_time( $c{DBI_sessionlifetime} );
+    my $expire_time = _get_expire_time( $c{'DBI_SessionLifetime'} );
 
     # Now we need to %-encode non-alphanumberics in the username so we
     # can stick it in the cookie safely.
@@ -664,9 +668,9 @@ sub authen_cred {
 
     # If we are using sessions, we create a new session for this login.
     my $session_id = $EMPTY_STRING;
-    if ( $c{DBI_sessionmodule} ne 'none' ) {
-        my $session =
-          _get_new_session( $r, $user, $auth_name, $c{DBI_sessionmodule},
+    if ( $c{'DBI_sessionmodule'} ne 'none' ) {
+        my $session
+            = _get_new_session( $r, $user, $auth_name, $c{'DBI_sessionmodule'},
             \@extra_data );
         $r->pnotes( $auth_name, $session );
         $session_id = $session->{_session_id};
@@ -677,29 +681,28 @@ sub authen_cred {
     # of the session key:
     my $current_time = _now_year_month_day_hour_minute_second;
     my $public_part  = "$enc_user:$current_time:$expire_time:$session_id";
-    $public_part .= $self->extra_session_info( $r, @credentials );
+    $public_part .= $class->extra_session_info( $r, @credentials );
 
     # Now we calculate the hash of this and the secret key and then
     # calculate the hash of *that* and the secret key again.
-    my $secretkey = $c{DBI_SecretKey};
+    my $secretkey = $c{'DBI_SecretKey'};
     if ( !defined $secretkey ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: didn't have the secret key for auth realm $auth_name",
+            "Apache2::AuthCookieDBI: didn't have the secret key for auth realm $auth_name",
             $r->uri
         );
         return;
     }
-    my $hash =
-      md5_hex( join q{:}, $secretkey,
+    my $hash = md5_hex( join q{:}, $secretkey,
         md5_hex( join q{:}, $public_part, $secretkey ) );
 
     # Now we add this hash to the end of the public part.
     my $session_key = "$public_part:$hash";
 
     # Now we encrypt this and return it.
-    my $encrypted_session_key =
-      _encrypt_session_key( $session_key, $secretkey, $auth_name,
-        $c{DBI_encryptiontype} );
+    my $encrypted_session_key
+        = _encrypt_session_key( $session_key, $secretkey, $auth_name,
+        $c{'DBI_EncryptionType'} );
     return $encrypted_session_key;
 }
 
@@ -707,7 +710,7 @@ sub authen_cred {
 # Take a session key and check that it is still valid; if so, return the user.
 
 sub authen_ses_key {
-    my ( $self, $r, $encrypted_session_key ) = @_;
+    my ( $class, $r, $encrypted_session_key ) = @_;
 
     my $auth_name = $r->auth_name;
 
@@ -715,55 +718,29 @@ sub authen_ses_key {
     my %c = _dbi_config_vars($r);
 
     # Get the secret key.
-    my $secret_key = $c{DBI_SecretKey};
+    my $secret_key = $c{'DBI_SecretKey'};
     if ( !defined $secret_key ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: didn't have the secret key from for auth realm $auth_name",
+            "Apache2::AuthCookieDBI: didn't have the secret key from for auth realm $auth_name",
             $r->uri
         );
         return;
     }
 
-    # Decrypt the session key.
-    my $session_key;
-    if ( $c{DBI_encryptiontype} eq 'none' ) {
-        $session_key = $encrypted_session_key;
-    }
-    else {
-
-        # Check that this looks like an encrypted hex-encoded string.
-        if ( $encrypted_session_key !~ $HEX_STRING_REGEX ) {
-            $r->log_error(
-"Apache2::AuthCookieDBI: encrypted session key $encrypted_session_key doesn't look like it's properly hex-encoded for auth realm $auth_name",
-                $r->uri
-            );
-            return;
-        }
-
-        my $cipher =
-          _get_cipher_type( $c{DBI_encryptiontype}, $auth_name, $secret_key );
-        if ( !$cipher ) {
-            $r->log_error(
-"Apache2::AuthCookieDBI: unknown encryption type $c{ DBI_encryptiontype } for auth realm $auth_name",
-                $r->uri
-            );
-            return;
-        }
-        $session_key = $cipher->decrypt_hex($encrypted_session_key);
-    }
+    my $session_key = $class->decrypt_session_key( $r, $c{'DBI_EncryptionType'},
+        $encrypted_session_key, $secret_key )
+        || return;
 
     # Break up the session key.
-    my (
-        $enc_user,   $issue_time,    $expire_time,
-        $session_id, @rest
-    ) = split $COLON_REGEX, $session_key;
+    my ( $enc_user, $issue_time, $expire_time, $session_id, @rest )
+        = split $COLON_REGEX, $session_key;
     my $hashed_string = pop @rest;
 
     # Let's check that we got passed sensible values in the cookie.
     ($enc_user) = _defined_or_empty($enc_user);
     if ( $enc_user !~ $PERCENT_ENCODED_STRING_REGEX ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: bad percent-encoded user '$enc_user' recovered from session ticket for auth_realm '$auth_name'",
+            "Apache2::AuthCookieDBI: bad percent-encoded user '$enc_user' recovered from session ticket for auth_realm '$auth_name'",
             $r->uri
         );
         return;
@@ -775,43 +752,43 @@ sub authen_ses_key {
     ($issue_time) = _defined_or_empty($issue_time);
     if ( $issue_time !~ $DATE_TIME_STRING_REGEX ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: bad issue time '$issue_time' recovered from ticket for user $user for auth_realm $auth_name",
+            "Apache2::AuthCookieDBI: bad issue time '$issue_time' recovered from ticket for user $user for auth_realm $auth_name",
             $r->uri
         );
         return;
     }
-    
+
     ($expire_time) = _defined_or_empty($expire_time);
     if ( $expire_time !~ $DATE_TIME_STRING_REGEX ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: bad expire time $expire_time recovered from ticket for user $user for auth_realm $auth_name",
+            "Apache2::AuthCookieDBI: bad expire time $expire_time recovered from ticket for user $user for auth_realm $auth_name",
             $r->uri
         );
         return;
     }
     if ( $hashed_string !~ $THIRTY_TWO_CHARACTER_HEX_STRING_REGEX ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: bad encrypted session_key $hashed_string recovered from ticket for user $user for auth_realm $auth_name",
+            "Apache2::AuthCookieDBI: bad encrypted session_key $hashed_string recovered from ticket for user $user for auth_realm $auth_name",
             $r->uri
         );
         return;
     }
 
     # If we're using a session module, check that their session exist.
-    if ( $c{DBI_sessionmodule} ne 'none' ) {
+    if ( $c{'DBI_sessionmodule'} ne 'none' ) {
         my %session;
         my $dbh = _dbi_connect($r) || return;
 
-        eval {
-            tie %session, $c{DBI_sessionmodule}, $session_id,
-              +{
+        my $tie_result = eval {
+            tie %session, $c{'DBI_sessionmodule'}, $session_id,
+                +{
                 Handle     => $dbh,
                 LockHandle => $dbh,
-              };
+                };
         };
-        if ($EVAL_ERROR) {
+        if ( ( !$tie_result ) || $EVAL_ERROR ) {
             $r->log_error(
-"Apache2::AuthCookieDBI: failed to tie session hash using session id $session_id for user $user for auth_realm $auth_name, error was $@",
+                "Apache2::AuthCookieDBI: failed to tie session hash using session id $session_id for user $user for auth_realm $auth_name, error was $@",
                 $r->uri
             );
             return;
@@ -837,7 +814,7 @@ sub authen_ses_key {
     # Compare it to the hash they gave us.
     if ( $new_hash ne $hashed_string ) {
         $r->log_error(
-"Apache2::AuthCookieDBI: hash '$hashed_string' in cookie did not match calculated hash '$new_hash' of contents for user $user for auth realm $auth_name",
+            "Apache2::AuthCookieDBI: hash '$hashed_string' in cookie did not match calculated hash '$new_hash' of contents for user $user for auth realm $auth_name",
             $r->uri
         );
         return;
@@ -846,7 +823,7 @@ sub authen_ses_key {
     # Check that their session hasn't timed out.
     if ( _now_year_month_day_hour_minute_second gt $expire_time ) {
         $r->log_error(
-"Apache:AuthCookieDBI: expire time $expire_time has passed for user $user for auth realm $auth_name",
+            "Apache:AuthCookieDBI: expire time $expire_time has passed for user $user for auth realm $auth_name",
             $r->uri
         );
         return;
@@ -856,24 +833,57 @@ sub authen_ses_key {
     # check that the issue time + the current (server-set) session lifetime
     # hasn't passed too (in case we issued long-lived session tickets
     # in the past that we want to get rid of). *** TODO ***
-    # if ( lc $c{ DBI_AlwaysUseCurrentSessionLifetime } eq 'on' ) {
+    # if ( lc $c{'DBI_AlwaysUseCurrentSessionLifetime'} eq 'on' ) {
 
     # They must be okay, so return the user.
     return $user;
 }
 
+sub decrypt_session_key {
+    my ( $class, $r, $encryptiontype, $encrypted_session_key, $secret_key )
+        = @_;
+
+    if ( $encryptiontype eq 'none' ) {
+        return $encrypted_session_key;
+    }
+
+    my $auth_name = $r->auth_name;
+
+    my $session_key;
+
+    # Check that this looks like an encrypted hex-encoded string.
+    if ( $encrypted_session_key !~ $HEX_STRING_REGEX ) {
+        $r->log_error(
+            "Apache2::AuthCookieDBI: encrypted session key '$encrypted_session_key' doesn't look like it's properly hex-encoded for auth realm $auth_name",
+            $r->uri
+        );
+        return;
+    }
+
+    my $cipher = _get_cipher_type( $encryptiontype, $auth_name, $secret_key );
+    if ( !$cipher ) {
+        $r->log_error(
+            "Apache2::AuthCookieDBI: unknown encryption type '$encryptiontype' for auth realm $auth_name",
+            $r->uri
+        );
+        return;
+    }
+    $session_key = $cipher->decrypt_hex($encrypted_session_key);
+    return $session_key;
+}
+
 #-------------------------------------------------------------------------------
 # Take a list of groups and make sure that the current remote user is a member
 # of one of them.
 
 sub group {
-    my ( $self, $r, $groups ) = @_;
+    my ( $class, $r, $groups ) = @_;
     my @groups = split( $WHITESPACE_REGEX, $groups );
 
     my $auth_name = $r->auth_name;
 
     # Get the configuration information.
-    my %c = _dbi_config_vars $r;
+    my %c = _dbi_config_vars($r);
 
     my $user = $r->user;
 
@@ -882,17 +892,17 @@ sub group {
 
     # Now loop through all the groups to see if we're a member of any:
     my $sth = $dbh->prepare_cached( <<"EOS" );
-SELECT $c{ DBI_groupuserfield }
-FROM $c{ DBI_groupstable }
-WHERE $c{ DBI_groupfield } = ?
-AND $c{ DBI_groupuserfield } = ?
+SELECT $c{'DBI_GroupUserField'}
+FROM $c{'DBI_GroupsTable'}
+WHERE $c{'DBI_GroupField'} = ?
+AND $c{'DBI_GroupUserField'} = ?
 EOS
     foreach my $group (@groups) {
         $sth->execute( $group, $user );
         return Apache2::Const::OK if ( $sth->fetchrow_array );
     }
     $r->log_error(
-"Apache2::AuthCookieDBI: user $user was not a member of any of the required groups @groups for auth realm $auth_name",
+        "Apache2::AuthCookieDBI: user $user was not a member of any of the required groups @groups for auth realm $auth_name",
         $r->uri
     );
     return Apache2::Const::HTTP_FORBIDDEN;
@@ -905,14 +915,14 @@ sub _get_expire_time {
     my $expire_time = $EMPTY_STRING;
 
     if ( $session_lifetime eq 'forever' ) {
-        $expire_time =
-          '9999-01-01-01-01-01'
-          ,    # expire time in a zillion years if it's forever.
-          return $expire_time;
+        $expire_time = '9999-01-01-01-01-01';
+
+        # expire time in a zillion years if it's forever.
+        return $expire_time;
     }
 
-    my ( $deltaday, $deltahour, $deltaminute, $deltasecond ) =
-      split $HYPHEN_REGEX, $session_lifetime;
+    my ( $deltaday, $deltahour, $deltaminute, $deltasecond )
+        = split $HYPHEN_REGEX, $session_lifetime;
 
     # Figure out the expire time.
     $expire_time = sprintf(
@@ -1009,10 +1019,12 @@ Apache2::Session(1)
 
 =over 2
 
-=item Add a proper set of regression tests!!! Easier said than done though.
+=item Improve test coverage.
 
 =item Refactor authen_cred() and authen_ses_key() into several smaller private methods.
 
+=item Refactor documentation.
+
 =back
 
 =cut
